'use strict';

const shared = require('@unocss-applet/shared');

function transformerApplet(options = {}) {
  const enable = options.enable ?? true;
  const layer = options.layer ?? "applet_shortcuts";
  if (options.unsupportedChars)
    shared.UNSUPPORTED_CHARS.push(...options.unsupportedChars);
  const ESCAPED_UNSUPPORTED_CHARS = shared.UNSUPPORTED_CHARS.map((char) => `\\${char}`);
  const charTestReg = new RegExp(`[${ESCAPED_UNSUPPORTED_CHARS.join("")}]`);
  const charReplaceReg = new RegExp(`[${ESCAPED_UNSUPPORTED_CHARS.join("")}]`, "g");
  return {
    name: "transformer-applet",
    enforce: "pre",
    async transform(s, id, ctx) {
      let code = s.toString();
      if (!enable)
        return;
      const { uno, tokens } = ctx;
      const { matched } = await uno.generate(code, { preflights: false });
      const replacements = Array.from(matched).filter((i) => charTestReg.test(i)).filter((i) => !i.includes("="));
      for (const replace of replacements) {
        let replaced = replace.replace(charReplaceReg, "_a_");
        replaced = shared.encodeNonLatin(replaced);
        while (replaced.startsWith("-"))
          replaced = replaced.slice(1);
        uno.config.shortcuts.push([replaced, replace, { layer }]);
        tokens.add(replaced);
        code = code.replace(new RegExp(escapeRegExp(replace), "g"), replaced);
      }
      s.overwrite(0, s.original.length, code);
    }
  };
}
function escapeRegExp(string) {
  return string.replace(/[.:%!#()[\]/,${}@+^&<>]/g, "\\$&");
}

module.exports = transformerApplet;
